/* split HIS format file to img files  */
/* usage is 'his_ave inputfile outputfile' */

#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<math.h>

#define INTEL
#define HIS_Header_Size 64
#define IMG_Header_Size	64

struct HIS_Header
{
	char			head[2];			/*0-1*/
	short			comment_length;		/*2-3*/
	short			width;				/*4-5*/
	short			height;				/*6-7*/
	short			x_offset;			/*8-9*/
	short			y_offset;			/*10-11*/
	short			type;				/*12-13*/
	unsigned short			n_image1;			/*14-17*/
	unsigned short			n_image2;			/*14-17*/
	short			reserve1;			/*18-19*/
	short			reserve2;			/*20-21*/
	double			time_stamp;			/*22-29*/
	long			maker;				/*30-33*/
	char			reserved[30];		/*34-63*/
	char			*comment;
};
typedef struct HIS_Header HISHeader;

struct IMG_Header{
	char			head[2];
	short			comment_length;
	short			width;
	short			height;
	short			x_offset;
	short			y_offset;
	short			type;
	char			reserved[63 - 13];
	char			*comment;
};
typedef struct IMG_Header IMGHeader;


short	*data;

static FILE *fi1, *fo;
FILE	*fi;

/*----------------------------------------------------------------------*/
int read_hishead(fname,his)
HISHeader	*his;
char *fname;
{
	int		j;
	FILE	*fi;

//open input files
	if((fi = fopen(fname,"rb")) == NULL){
		printf("can not open %s for input\n", fname);
		free(his->comment); 
		return(-10);
	}
// read comment and image data from file1
	if (fread(his, sizeof(char), HIS_Header_Size, fi) != HIS_Header_Size){
		fprintf(stderr, "EOF in input file (header)\n"); free(his->comment); return(-1);
	}
	if (strncmp(his->head, "IM", 2)){
		fclose(fi); free(his->comment);
		fprintf(stderr, "This is not HIS image file.\n"); 
		return(-11);
	}

	his->comment=(char *)malloc(his->comment_length+1);
	if (his->comment == NULL){
		fclose(fi); free(his->comment); 
		fprintf(stderr, "Connot allocate memory (HIS_Header.comment)\n"); 
		return(-12);
	}
	if (fread(his->comment, sizeof(char), his->comment_length, fi) != his->comment_length){ 
		fclose(fi); free(his->comment); 
		fprintf(stderr, "EOF found (comment)\n"); 
		return(-13);
	}
	his->comment[his->comment_length] = '\0';

// check for correct reading of HIS file
//printf("-------------------------------\n");
//printf("%s\n", his->head);
//printf("%d\t", his->comment_length);
//printf("%d\t", his->width);
//printf("%d\t", his->height);
//printf("%d\t", his->x_offset);
//printf("%d\t", his->y_offset);
//printf("%d\t", his->type);
//printf("%d\t", his->n_image1);			/*14-17*/
//printf("%ld\t", his->n_image2*65536);			/*14-17*/
//printf("%d\t", his->reserve1);			/*18-19*/
//printf("%d\t", his->reserve2);			/*20-21*/
//printf("%lf\t", his->time_stamp);			/*22-29*/
//printf("%ld\t", his->maker);				/*30-33*/
//printf("%s\n", his->reserved);	/*34-64*/
//printf("%s\n", his->comment);
//printf("-------------------------------\n");
	
	
	fclose(fi);
	return(0);
}

/*----------------------------------------------------------------------*/
int read_his(his,img)
HISHeader	*his;
IMGHeader	*img;
{
	int		j;

// read comment and image data from file1
	if (fread(his, sizeof(char), HIS_Header_Size, fi) != HIS_Header_Size){
		fprintf(stderr, "EOF in input file (header)\n"); free(his->comment); return(-1);
	}
	if (strncmp(his->head, "IM", 2)){
		fclose(fi); free(his->comment);
		fprintf(stderr, "This is not HIS image file.\n"); 
		return(-11);
	}

	his->comment=(char *)malloc(his->comment_length+1);
	if (his->comment == NULL){
		fclose(fi); free(his->comment); 
		fprintf(stderr, "Connot allocate memory (HIS_Header.comment)\n"); 
		return(-12);
	}
	if (fread(his->comment, sizeof(char), his->comment_length, fi) != his->comment_length){ 
		fclose(fi); free(his->comment); 
		fprintf(stderr, "EOF found (comment)\n"); 
		return(-13);
	}
	his->comment[his->comment_length] = '\0';

	if ((j = fread(data, sizeof(short), his->width*his->height, fi)) != his->width*his->height){
		fclose(fi); free(his->comment); 
		fprintf(stderr, " Error reading at %d\n",  j);
		return(1);
	}

	sprintf(img->head,"IM");
//	img->comment_length=his->comment_length;
	img->comment_length=0;
	img->width=his->width;
	img->height=his->height;
	img->x_offset=his->x_offset;
	img->y_offset=his->y_offset;
	img->type=his->type;
//	sprintf(img->reserved,"");
//	sprintf(img->comment,"%s",his->comment);

		return(0);
	}

/*----------------------------------------------------------------------*/


int	main(argc,argv)
int		argc;
char	*argv[];
{
	short	*image1, *outimg;
	unsigned long	*lngimg;
	long	i, j, jj, k;
	char	readfile[20], outfile[20];
	char	*cmt;
	short	clg;
	unsigned long	i_end;

	HISHeader		his;
	IMGHeader		img;

	if(argc != 3){   // when bad number of parameters
		printf("Usage is 'his_ave inputfile outputfile'\n"); exit(-1);
	}

	sprintf(readfile, ("%s"), argv[1]);

	if ((j = read_hishead(readfile, &his)) != 0){
		printf("something wrong -- return value is %d", j);
		free(his.comment); 
		return(-1);
	}
	i_end=his.n_image1+65536*his.n_image2;

	printf("read %s \t %ld images \n",readfile, i_end);

// initialize
	lngimg = (unsigned long *) malloc(his.width * his.height * sizeof(unsigned long));
	for(jj=0;jj<his.width * his.height;++jj) *(lngimg+jj)=0;
	
	data=(short *)malloc(his.width * his.height * sizeof(short));
//open input files
	if((fi = fopen(argv[1],"rb")) == NULL){
		printf("can not open %s for input\n", argv[1]);
		return(-10);
	}
	for(k=0;k<i_end;++k){
		printf("%d\n",k);
// read input images
		if ((j = read_his(&his, &img)) != 0){
			printf("something wrong -- return value is %d", i);
			free(his.comment); 
			return(-1);
		}
		
		for(jj=0;jj<img.width * img.height;++jj) *(lngimg+jj)=*(lngimg+jj)+*(data+jj);
	}

	outimg = (unsigned short *) malloc(img.width * img.height * sizeof(unsigned short));
	for(jj=0;jj<img.width * img.height;++jj) *(outimg+jj)=*(lngimg+jj)/i_end;

	sprintf(outfile, ("%s"), argv[2]);

// open output file
	if((fo  = fopen(outfile,"wb")) == NULL){ // file3 open
		printf("can not open file3 for output\n"); return(-1);
	}
// write dummy header to file3
	if (fwrite(&img, sizeof(char), IMG_Header_Size, fo) != IMG_Header_Size){
		fprintf(stderr, "something wrong in %d (header3)\n", k);
		exit(-1);
	}
	if (fwrite(img.comment, sizeof(char), img.comment_length, fo) != img.comment_length){
		fprintf(stderr, "something wrong in comment3 of %d (header)\n", k);
		exit(-1);
	}

// write comment and dark image to file3
	if (i = fwrite(outimg , sizeof(short), img.width*img.height, fo) > img.width*img.height*sizeof(short)){
		fprintf(stderr, "something wrong in data3 area %d  %d\n", i,  img.width*img.height);
		exit(-1);
	}
// close files
	fclose(fo);

	printf("output to %s.\n",outfile);
	free(outimg);
	free(lngimg);
	free(data);
	
//finish!
	free(his.comment);
	free(img.comment);
	fclose(fi);
	return(0);
}
